"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.rateLimiterMiddleware = exports.authTokenMiddleware = exports.trustedUrlMiddleware = exports.asyncMiddleware = void 0;
const boom = require("@hapi/boom");
const security_1 = require("../config/security");
const asyncMiddleware = (fn) => (req, res, next) => {
    Promise.resolve(fn(req, res, next)).catch((err) => {
        if (!err.isBoom) {
            return next(boom.badImplementation(err));
        }
        next(err);
    });
};
exports.asyncMiddleware = asyncMiddleware;
const trustedUrlMiddleware = (req, res, next) => {
    const queryUrl = req.query.url;
    if (queryUrl && !(queryUrl.startsWith('http://') || queryUrl.startsWith('https://'))) {
        return next(boom.forbidden('Forbidden query url protocol'));
    }
    next();
};
exports.trustedUrlMiddleware = trustedUrlMiddleware;
const authTokenMiddleware = (config) => {
    return (req, res, next) => {
        const headerToken = req.header('X-Auth-Token');
        if (headerToken === undefined || !(0, security_1.isAuthTokenValid)(config, headerToken)) {
            return next(boom.unauthorized('Unauthorized request'));
        }
        next();
    };
};
exports.authTokenMiddleware = authTokenMiddleware;
const rateLimiterMiddleware = (rateLimiter) => {
    return async (req, res, next) => {
        const rateLimiterKey = req.header('X-Tenant-ID') || req.ip;
        if (rateLimiterKey === undefined) {
            return next(boom.badRequest('Missing X-Tenant-ID header to use rate limiter'));
        }
        try {
            await rateLimiter.consume(rateLimiterKey);
            next();
        }
        catch (err) {
            res.set('Retry-After', String(Math.ceil(err.msBeforeNext / 1000)));
            res.status(429).send('Too Many Requests');
        }
    };
};
exports.rateLimiterMiddleware = rateLimiterMiddleware;
//# sourceMappingURL=middlewares.js.map